==> oai-harvester-1.0.2-20200520.141245-15-sources.jar: org/gcube/data/publishing/gFeed/collectors/oai/OAIClient.java <==
package org.gcube.data.publishing.gFeed.collectors.oai;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collection;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;
import javax.xml.bind.JAXB;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;

import org.gcube.data.publishing.gFeed.collectors.oai.model.CommunicationException;
import org.gcube.data.publishing.gFeed.collectors.oai.model.DCRecordMetadata;
import org.gcube.data.publishing.gFeed.collectors.oai.model.MetadataHolder;
import org.gcube.data.publishing.gFeed.collectors.oai.model.OAIInteractionException;
import org.gcube.data.publishing.gFeed.collectors.oai.model.OAIMetadata;
import org.gcube.data.publishing.gFeed.collectors.oai.model.OAIRecord;
import org.gcube.data.publishing.gFeed.collectors.oai.model.OAI_PMH;
import org.gcube.data.publishing.gFeed.collectors.oai.model.OAI_PMH.Token;
import org.glassfish.jersey.client.ClientProperties;

import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@RequiredArgsConstructor
public class OAIClient {

	private static JAXBContext jaxbContext=null;
	private static final int MAX_ATTEMPTS=3;
	private static final long DELAY_FACTOR=1000;
	
	
	private static synchronized JAXBContext getContext() throws JAXBException {
		if(jaxbContext==null)
			jaxbContext = JAXBContext.newInstance(OAIRecord.class,
					MetadataHolder.class,
					OAIMetadata.class,
					DCRecordMetadata.class,
					OAI_PMH.class);
		return jaxbContext;
	}



	public static final String DC_METADATA_PREFIX="oai_dc";

	@NonNull
	private String baseUrl;


	Client client;

	private synchronized Client getWebClient() {
		if(client==null) {
			client = ClientBuilder.newClient()
					.property(ClientProperties.SUPPRESS_HTTP_COMPLIANCE_VALIDATION, true);
		}		
		return client;
	}



	public Collection<OAIRecord> getAll(String metadataPrefix) throws JAXBException, OAIInteractionException{
		ArrayList<OAIRecord> toReturn=new ArrayList<OAIRecord>();

		String resumptionToken=null;

		// call & iterate
		boolean isComplete=false;
		int currentAttempt=1;
		while(!isComplete) {
			try {
				WebTarget target=getWebClient().target(baseUrl).
						queryParam("verb","ListRecords");

				if(resumptionToken==null)
					target=target.queryParam("metadataPrefix",metadataPrefix);
				else
					target=target.queryParam("resumptionToken", resumptionToken);


				Response resp=target.request("application/xml").get();

				OAI_PMH msg=check(resp);

				if(msg.isError()) throw new OAIInteractionException(msg.getError().getCode()+ " : "+msg.getError().getMessage());
				//No errors, thus reset attempt counter
				currentAttempt=1;

				toReturn.addAll(msg.getResponseRecords().getRecords());
				log.debug("Parsed "+toReturn.size()+" records so far.");

				Token t=msg.getResponseRecords().getResumptionToken();
				log.debug("Obtained token : "+t);
				
				if(t!=null && t.getId()!=null && !t.getId().isEmpty()) {
					resumptionToken=t.getId();
				}else isComplete=true; //no token = completion
				
//			}catch(CommunicationException e) {
//				log.warn("Received communication error "+e.getMessage());
//				log.debug("Current attempt number = "+currentAttempt," max attempt Number = "+MAX_ATTEMPTS+", attempts delay factor =  ");
//				isComplete=currentAttempt>MAX_ATTEMPTS;
//				try {
//					Thread.sleep(currentAttempt*DELAY_FACTOR);
//				} catch (InterruptedException e1) {}
//				currentAttempt++;
//				
//			}catch(OAIInteractionException e) {
//				log.warn("Remote OAI "+baseUrl+" didn't accept request ",e);				
//				log.debug("Current attempt number = "+currentAttempt," max attempt Number = "+MAX_ATTEMPTS+", attempts delay factor =  ");
//				isComplete=currentAttempt>MAX_ATTEMPTS;
//				try {
//					Thread.sleep(currentAttempt*DELAY_FACTOR);
//				} catch (InterruptedException e1) {}
//				currentAttempt++;
			}catch(Throwable t) {
//				throw new OAIInteractionException("Unexpected error while harvesting "+baseUrl,t); 
				log.warn("Unexpected ERROR	 "+t.getMessage());
				log.debug("Current attempt number = "+currentAttempt," max attempt Number = "+MAX_ATTEMPTS+", attempts delay factor =  ");
				isComplete=currentAttempt>MAX_ATTEMPTS;
				try {
					Thread.sleep(currentAttempt*DELAY_FACTOR);
				} catch (InterruptedException e1) {}
				currentAttempt++;
			}
		}
		log.trace("Obtained "+toReturn.size()+" from "+baseUrl);
		return toReturn;
	}


	private void retry() {
		
	}

	private static OAI_PMH check(Response resp) throws JAXBException, CommunicationException {
		if(resp.getStatus()<200||resp.getStatus()>=300) {
			// exception
			throw new CommunicationException("Received error message. STATUS "+resp.getStatus()+ ", message : "+resp.readEntity(String.class));
		}else {


			String respString=resp.readEntity(String.class);
			Unmarshaller jaxbUnmarshaller = getContext().createUnmarshaller();
			OAI_PMH obj=(OAI_PMH) jaxbUnmarshaller.unmarshal(new StringReader(respString));

			return obj;

			//			 OAI_PMH response = (OAI_PMH) jaxbUnmarshaller.unmarshal(
			//					 new StreamSource(new StringReader(respString)));

		}

	}
}

==> oai-harvester-1.0.2-20200526.104605-19-sources.jar: org/gcube/data/publishing/gFeed/collectors/oai/OAIClient.java <==
package org.gcube.data.publishing.gFeed.collectors.oai;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collection;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.gcube.data.publishing.gFeed.collectors.oai.model.CommunicationException;
import org.gcube.data.publishing.gFeed.collectors.oai.model.DCRecordMetadata;
import org.gcube.data.publishing.gFeed.collectors.oai.model.MetadataHolder;
import org.gcube.data.publishing.gFeed.collectors.oai.model.OAIInteractionException;
import org.gcube.data.publishing.gFeed.collectors.oai.model.OAIMetadata;
import org.gcube.data.publishing.gFeed.collectors.oai.model.OAIRecord;
import org.gcube.data.publishing.gFeed.collectors.oai.model.OAI_PMH;
import org.gcube.data.publishing.gFeed.collectors.oai.model.OAI_PMH.Token;
import org.glassfish.jersey.client.ClientProperties;

import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@RequiredArgsConstructor
public class OAIClient {

	private static JAXBContext jaxbContext=null;
	private static final int MAX_ATTEMPTS=3;
	private static final long DELAY_FACTOR=1000;
	
	
	private static synchronized JAXBContext getContext() throws JAXBException {
		if(jaxbContext==null)
			jaxbContext = JAXBContext.newInstance(OAIRecord.class,
					MetadataHolder.class,
					OAIMetadata.class,
					DCRecordMetadata.class,
					OAI_PMH.class);
		return jaxbContext;
	}



	public static final String DC_METADATA_PREFIX="oai_dc";

	@NonNull
	private String baseUrl;

	@NonNull
	@Setter
	private Integer maxItems=-1;
	
	
	Client client;

	private synchronized Client getWebClient() {
		if(client==null) {
			client = ClientBuilder.newClient()
					.property(ClientProperties.SUPPRESS_HTTP_COMPLIANCE_VALIDATION, true);
		}		
		return client;
	}



	public Collection<OAIRecord> getAll(String metadataPrefix) throws JAXBException, OAIInteractionException{
		ArrayList<OAIRecord> toReturn=new ArrayList<OAIRecord>();

		String resumptionToken=null;

		// call & iterate
		boolean isComplete=false;
		int currentAttempt=1;
		while(!isComplete) {
			try {
				WebTarget target=getWebClient().target(baseUrl).
						queryParam("verb","ListRecords");

				if(resumptionToken==null)
					target=target.queryParam("metadataPrefix",metadataPrefix);
				else
					target=target.queryParam("resumptionToken", resumptionToken);


				Response resp=target.request("application/xml").get();

				OAI_PMH msg=check(resp);

				if(msg.isError()) throw new OAIInteractionException(msg.getError().getCode()+ " : "+msg.getError().getMessage());
				//No errors, thus reset attempt counter
				currentAttempt=1;

				toReturn.addAll(msg.getResponseRecords().getRecords());
				log.debug("Parsed "+toReturn.size()+" records so far.");

				Token t=msg.getResponseRecords().getResumptionToken();
				log.debug("Obtained token : "+t);
				
				if(t!=null && t.getId()!=null && !t.getId().isEmpty()) {
					resumptionToken=t.getId();
				}else isComplete=true; //no token = completion
				
				
				
				//Using limit 
				if(maxItems>0 && toReturn.size()>=maxItems) {
					log.warn("MAX ITEMS LIMIT REACHED : "+toReturn.size()+" / "+maxItems);
					isComplete=true;
				}
				
				
				
//			}catch(CommunicationException e) {
//				log.warn("Received communication error "+e.getMessage());
//				log.debug("Current attempt number = "+currentAttempt," max attempt Number = "+MAX_ATTEMPTS+", attempts delay factor =  ");
//				isComplete=currentAttempt>MAX_ATTEMPTS;
//				try {
//					Thread.sleep(currentAttempt*DELAY_FACTOR);
//				} catch (InterruptedException e1) {}
//				currentAttempt++;
//				
//			}catch(OAIInteractionException e) {
//				log.warn("Remote OAI "+baseUrl+" didn't accept request ",e);				
//				log.debug("Current attempt number = "+currentAttempt," max attempt Number = "+MAX_ATTEMPTS+", attempts delay factor =  ");
//				isComplete=currentAttempt>MAX_ATTEMPTS;
//				try {
//					Thread.sleep(currentAttempt*DELAY_FACTOR);
//				} catch (InterruptedException e1) {}
//				currentAttempt++;
			}catch(Throwable t) {
//				throw new OAIInteractionException("Unexpected error while harvesting "+baseUrl,t); 
				log.warn("Unexpected ERROR	 "+t.getMessage());
				log.debug("Current attempt number = "+currentAttempt," max attempt Number = "+MAX_ATTEMPTS+", attempts delay factor =  ");
				isComplete=currentAttempt>MAX_ATTEMPTS;
				try {
					Thread.sleep(currentAttempt*DELAY_FACTOR);
				} catch (InterruptedException e1) {}
				currentAttempt++;
			}
		}
		log.trace("Obtained "+toReturn.size()+" from "+baseUrl);
		return toReturn;
	}


	private void retry() {
		
	}

	private static OAI_PMH check(Response resp) throws JAXBException, CommunicationException {
		if(resp.getStatus()<200||resp.getStatus()>=300) {
			// exception
			throw new CommunicationException("Received error message. STATUS "+resp.getStatus()+ ", message : "+resp.readEntity(String.class));
		}else {


			String respString=resp.readEntity(String.class);
			Unmarshaller jaxbUnmarshaller = getContext().createUnmarshaller();
			OAI_PMH obj=(OAI_PMH) jaxbUnmarshaller.unmarshal(new StringReader(respString));

			return obj;

			//			 OAI_PMH response = (OAI_PMH) jaxbUnmarshaller.unmarshal(
			//					 new StreamSource(new StringReader(respString)));

		}

	}
}
